import numpy as np
import matplotlib.pyplot as plt
import math

#----------------
# データ入力
#----------------

#地震加速度データの読み込み
ydd = np.zeros(2689)
file = open('ElCentroNS.txt','r')
dteq = 0.02                #地震加速度の時間刻み
for i  in range(2689):
    ydd[i] = file.readline()
file.close()

#減衰定数の入力
hs = input("h = ")
hs = float(hs)
#解析時間の入力
tm = input("tmax(sec) = ")
tm = float(tm)
#固有周期の最大値
Tpmax = input("Tpmax(sec) = ")
Tpmax = float(Tpmax)
#固有周期間隔
dTp = input("dTp = ")
dTp = float(dTp)

x1 = np.arange(0,Tpmax,dTp)
x2 = np.arange(0,Tpmax,dTp)
x3 = np.arange(0,Tpmax,dTp)
nt = int(Tpmax/dTp)
y1 = np.zeros(nt)
y2 = np.zeros(nt)
y3 = np.zeros(nt)

# ------------------------------------------------------------
#     応答スペクトルの計算
# ------------------------------------------------------------
fileout = open('output2.txt','w')

for itp in range(500):
    Tp = dTp*float(itp+1)
    if Tp > Tpmax + dTp/2.0 : break
    xmax = 0.0   # 最大変位
    vmax = 0.0   # 最大速度
    amax = 0.0   # 最大加速度（地動加速度も含む）

    # --- デュアメル積分法による応答解析
    tn = 0.0     #時間
    dt = dteq    #時間増分
    xn = 0.0     #変位
    xdn = 0.0    #速度
    xddn = 0.0   #加速度
    accn = xddn + ydd[0]

    xi = np.zeros(2)
    et = np.zeros(2)
    xi[0] = -0.57735026918963   #ガウス積分の選点1
    xi[1] = 0.57735026918963    #ガウス積分の選点2
    et[0] = 1.0                 #ガウス積分の重み1
    et[1] = 1.0                 #ガウス積分の重み2
    pi = 3.141592654
    om = 2.0 * pi / Tp                   #ω
    omd = om * math.sqrt(1.0 - hs ** 2)  #ω'
    Atn = 0.0                            #A(t - Dt)
    Btn = 0.0                            #B(t - Dt)

    for n in range(2688):
        tn1 = tn + dt     #tn+1
        if tn1 > tm + dt/2.0 : break
        # --- A(t), B(t)の計算
        Ata = 0.0
        Bta = 0.0
        for i in range(2):    #ガウス積分
            tui = tn1 - (dt / 2.0) + (dt / 2.0) * xi[i]
            wti = (dt / 2.0) * et[i]
            yddti = ydd[n] + (ydd[n + 1]- ydd[n]) / dt * (tui - tn)
            Ata = Ata - (1.0 / omd) * wti * yddti * math.exp(-hs * om * (tn1 - tui)) * math.cos(omd * tui)
            Bta = Bta - (1.0 / omd) * wti * yddti * math.exp(-hs * om * (tn1 - tui)) * math.sin(omd * tui)
        Atn1 = Atn * math.exp(-hs * om * dt) + Ata   # A(t)
        Btn1 = Btn * math.exp(-hs * om * dt) + Bta   # B(t)
        # ---
        xn = Atn1 * math.sin(omd * tn1) - Btn1 * math.cos(omd * tn1)
        xdn = omd * (Atn1 * math.cos(omd * tn1) + Btn1 * math.sin(omd * tn1)) - hs * om * xn
        xddn = -2.0 * hs * om * xdn - om ** 2 * xn - ydd[n + 1]
        #---
        accn = xddn + ydd[n+1]

        if abs(xn) > xmax : xmax = abs(xn)
        if abs(xdn) > vmax : vmax = abs(xdn)
        if abs(accn) > amax : amax = abs(accn)

        tn = tn1    # 時間の更新
        Atn = Atn1  # A(t - Dt) の更新
        Btn = Btn1  # B(t - Dt) の更新

    s = str('%10.3f' % Tp) + str('  %10.3e' % xmax) + str('  %10.3e' % vmax) + str('  %10.3e' % amax)
    fileout.write(s)
    fileout.write('\n')
    
    y1[itp] = xmax
    y2[itp] = vmax
    y3[itp] = amax

fileout.close()

fig, (ax1, ax2, ax3) = plt.subplots(ncols=3, figsize=(14,4))

ax1.plot(x1,y1)
ax1.set_xlabel("t")
ax1.set_ylabel("d_max")

ax2.plot(x2,y2)
ax2.set_xlabel("t")
ax2.set_ylabel("v_max")

ax3.plot(x3,y3)
ax3.set_xlabel("t")
ax3.set_ylabel("a_max")

fig.show