import numpy as np
import matplotlib.pyplot as plt
import math

#----------------
# データ入力
#----------------

#地震加速度データの読み込み
ydd = np.zeros(3000)
file = open('ElCentroNS.txt','r')
dteq = 0.02                #地震加速度の時間刻み
for i in range(2689):
    ydd[i] = file.readline()
file.close()

#質量の入力
ms = input("m = ")
ms = float(ms)
#剛性の入力
ks = input("k = ")
ks = float(ks)
#減衰定数の入力
hs = input("h = ")
hs = float(hs)
cs = hs*2.0*math.sqrt(ms*ks)
#解析時間の入力
tm = input("tmax(sec) = ")
tm = float(tm)

ntm = int(tm)
x1 = np.arange(0,ntm,0.02)
x2 = np.arange(0,ntm,0.02)
nt = int(tm/0.02)
y1 = np.zeros(nt)
y2 = np.zeros(nt)

# ------------------------------------------------------------
#     応答解析（平均加速度法）
# ------------------------------------------------------------
fileout = open('output3.txt','w')

tn = 0.0     #時間
dt = dteq    #時間増分
xn = 0.0     #変位
xdn = 0.0    #速度
xddn = 0.0   #加速度
accn = xddn + ydd[0]

s = str('%10.3f' % tn) + str('  %10.3e' % xn) + str('  %10.3e' % xdn) + str('  %10.3e' % accn)
fileout.write(s)
fileout.write('\n')

for n in range(2688):
    tn1 = tn + dt     #tn+1
    if tn1 > tm + dt/2.0 : break
    dydd = ydd[n + 1] - ydd[n]
    dx = (ms * (-dydd + (4.0/dt)*xdn + 2.0*xddn) + 2.0*cs*xdn)/(ks + (2.0/dt) * cs + (4.0/dt**2)*ms)
    dxd = (2.0/dt)*dx - 2.0*xdn
    dxdd = (4.0/dt**2)*dx - (4.0/dt)*xdn - 2.0 * xddn
    xn = xn + dx           #xn + 1
    xdn = xdn + dxd        #xdn + 1
    xddn = xddn + dxdd     #xddn + 1
    accn = xddn + ydd[n+1]
    
    y2[n] = xn

    s = str('%10.3f' % tn1) + str('  %10.3e' % xn) + str('  %10.3e' % xdn) + str('  %10.3e' % accn)
    fileout.write(s)
    fileout.write('\n')
    tn = tn1    # 時間の更新

fileout.close()

fig, (ax1, ax2) = plt.subplots(nrows=2, figsize=(10,8))

for i in range(nt):
    y1[i] = ydd[i]

ax1.plot(x1,y1)
ax1.set_xlabel("t")
ax1.set_ylabel("ydd")

ax2.plot(x2,y2)
ax2.set_xlabel("t")
ax2.set_ylabel("x")