import numpy as np
import matplotlib.pyplot as plt
import math

#----------------
# データ入力
#----------------

#地震加速度データの読み込み
ydd = np.zeros(2689)
file = open('ElCentroNS.txt','r')
dteq = 0.02                #地震加速度の時間刻み
for i  in range(2689):
    ydd[i] = file.readline()
file.close()

#質量の入力
ms = input("m = ")
ms = float(ms)
#剛性の入力
ks = input("k = ")
ks = float(ks)
#減衰定数の入力
hs = input("h = ")
hs = float(hs)
cs = hs*2.0*math.sqrt(ms*ks)
#解析時間の入力
tm = input("tmax(sec) = ")
tm = float(tm)

ntm = int(tm)
x1 = np.arange(0,ntm,0.02)
x2 = np.arange(0,ntm,0.02)
nt = int(tm/0.02)
y1 = np.zeros(nt)
y2 = np.zeros(nt)

# ------------------------------------------------------------
#     応答解析（デュアメル積分法）
# ------------------------------------------------------------
fileout = open('output1.txt','w')

tn = 0.0     #時間
dt = dteq    #時間増分
xn = 0.0     #変位
xdn = 0.0    #速度
xddn = 0.0   #加速度
accn = xddn + ydd[0]

s = str('%10.3f' % tn) + str('  %10.3e' % xn) + str('  %10.3e' % xdn) + str('  %10.3e' % accn)
fileout.write(s)
fileout.write('\n')

xi = np.zeros(2)
et = np.zeros(2)
xi[0] = -0.57735026918963   #ガウス積分の選点1
xi[1] = 0.57735026918963    #ガウス積分の選点2
et[0] = 1.0                 #ガウス積分の重み1
et[1] = 1.0                 #ガウス積分の重み2
om = math.sqrt(ks / ms)              #ω
omd = om * math.sqrt(1.0 - hs ** 2)  #ω'
Atn = 0.0                            #A(t - Dt)
Btn = 0.0                            #B(t - Dt)

for n in range(2688):
    tn1 = tn + dt     #tn+1
    if tn1 > tm + dt/2.0 : break
    # --- A(t), B(t)の計算
    Ata = 0.0
    Bta = 0.0
    for i in range(2):    #ガウス積分
        tui = tn1 - (dt / 2.0) + (dt / 2.0) * xi[i]
        wti = (dt / 2.0) * et[i]
        yddti = ydd[n] + (ydd[n + 1]- ydd[n]) / dt * (tui - tn)
        Ata = Ata - (1.0 / omd) * wti * yddti * math.exp(-hs * om * (tn1 - tui)) * math.cos(omd * tui)
        Bta = Bta - (1.0 / omd) * wti * yddti * math.exp(-hs * om * (tn1 - tui)) * math.sin(omd * tui)
    Atn1 = Atn * math.exp(-hs * om * dt) + Ata   # A(t)
    Btn1 = Btn * math.exp(-hs * om * dt) + Bta   # B(t)
    # ---
    xn = Atn1 * math.sin(omd * tn1) - Btn1 * math.cos(omd * tn1)
    xdn = omd * (Atn1 * math.cos(omd * tn1) + Btn1 * math.sin(omd * tn1)) - hs * om * xn
    xddn = -2.0 * hs * om * xdn - om ** 2 * xn - ydd[n + 1]
    #---
    accn = xddn + ydd[n+1]
    y2[n] = xn
    s = str('%10.3f' % tn1) + str('  %10.3e' % xn) + str('  %10.3e' % xdn) + str('  %10.3e' % accn)
    fileout.write(s)
    fileout.write('\n')
    tn = tn1    # 時間の更新
    Atn = Atn1  # A(t - Dt) の更新
    Btn = Btn1  # B(t - Dt) の更新

fileout.close()

fig, (ax1, ax2) = plt.subplots(nrows=2, figsize=(10,8))

for i in range(nt):
    y1[i] = ydd[i]

ax1.plot(x1,y1)
ax1.set_xlabel("t")
ax1.set_ylabel("ydd")

ax2.plot(x2,y2)
ax2.set_xlabel("t")
ax2.set_ylabel("x")

fig.show