import numpy as np
import math

#----------------
# データ入力
#----------------

#地震加速度データの読み込み
ydd = np.zeros(2689)
file = open('ElCentroNS.txt','r')
dteq = 0.02                #地震加速度の時間刻み
for i  in range(2689):
    ydd[i] = file.readline()
file.close()

#層数の入力
Nn = input("N = ")
Nn = int(Nn)

mi = np.zeros(Nn)         # 各層の質量
ki = np.zeros(Nn)         # 各層の剛性
ms = np.zeros((Nn,Nn))    # 質量マトリクス
ks = np.zeros((Nn,Nn))    # 剛性マトリクス
cs = np.zeros((Nn,Nn))    # 減衰マトリクス
kb = np.zeros((Nn,Nn))    # 剛性方程式の剛性マトリクス
fb = np.zeros(Nn)         # 剛性方程式の外力ベクトル
dx = np.zeros(Nn)         # 変位増分ベクトル
xn = np.zeros(Nn)         # 変位ベクトル
xdn = np.zeros(Nn)        # 速度ベクトル
xddn = np.zeros(Nn)       # 加速度ベクトル

for i in range(Nn):
    print('No.=',i+1)
    #質量の入力
    mi[i] = input("m = ")
    #剛性の入力
    ki[i] = input("k = ")

#減衰定数の入力
hc = input("2h1/om = ")
hc = float(hc)
#解析時間の入力
tm = input("tmax(sec) = ")
tm = float(tm)

# --- 質量マトリクス[M]
for i in range(Nn):
    ms[i][i] = mi[i]
# --- 剛性マトリクス[K]
for i in range(Nn-1):
    ks[i][i] = ki[i] + ki[i+1]
    ks[i][i+1] = -ki[i+1]
    ks[i+1][i] = -ki[i+1]
ks[Nn-1][Nn-1] = ki[Nn-1]
# --- 減衰マトリクス[C]
for i in range(Nn):
    for j in range(Nn):
        cs[i][j] = hc*ks[i][j]

# ------------------------------------------------------------
#     応答解析（平均加速度法）
# ------------------------------------------------------------
fileout = open('output4.txt','w')

tn = 0.0     #時間
dt = dteq    #時間増分
accn = xddn[Nn-1] + ydd[0]

s = str('%10.3f' % tn) + str('  %10.3e' % xn[Nn-1]) + str('  %10.3e' % xdn[Nn-1]) + str('  %10.3e' % accn)
fileout.write(s)
fileout.write('\n')

for n in range(2688):
    tn1 = tn + dt     #tn+1
    if tn1 > tm + dt/2.0 : break
    dydd = ydd[n + 1] - ydd[n]
    # ---- [kb]
    for i in range(Nn):
        for j in range(Nn):
            kb[i][j] = ks[i][j] + (2.0/dt)*cs[i][j] + (4.0/dt**2)*ms[i][j]
    # ---- [fb]
    for i in range(Nn):
        fb[i] = 0.0
        for j in range(Nn):
            fb[i] = fb[i] + ms[i][j]*(-dydd + (4.0/dt)*xdn[j] + 2.0*xddn[j]) + 2.0*cs[i][j]*xdn[j]
    # ---- 連立方程式
    dx = np.linalg.solve(kb, fb)

    for i in range(Nn):
        dxi = dx[i]
        dxdi = (2.0/dt)*dxi - 2.0*xdn[i]
        dxddi = (4.0/dt**2)*dxi - (4.0/dt)*xdn[i] - 2.0*xddn[i]
        xn[i] = xn[i] + dxi
        xdn[i] = xdn[i] + dxdi
        xddn[i] = xddn[i] + dxddi

    accn = xddn[Nn-1] + ydd[n+1]
    s = str('%10.3f' % tn1) + str('  %10.3e' % xn[Nn-1]) + str('  %10.3e' % xdn[Nn-1]) + str('  %10.3e' % accn)
    fileout.write(s)
    fileout.write('\n')
    tn = tn1    # 時間の更新

fileout.close()